<?php
/**
 * Ce fichier contient les fonctions internes destinées à standardiser les données météorologiques.
 *
 * @package SPIP\RAINETTE\MASHUP
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

if (!defined('_RAINETTE_REGEXP_LIEU_IP')) {
	/**
	 * Regexp permettant de reconnaître un lieu au format adresse IP.
	 */
	define('_RAINETTE_REGEXP_LIEU_IP', '#(?:\d{1,3}\.){3}\d{1,3}#');
}
if (!defined('_RAINETTE_REGEXP_LIEU_COORDONNEES')) {
	/**
	 * Regexp permettant de reconnaître un lieu au format coordonnées géographiques latitude,longitude.
	 */
	define('_RAINETTE_REGEXP_LIEU_COORDONNEES', '#([\-\+]?\d+(?:\.\d+)?)\s*,\s*([\-\+]?\d+(?:\.\d+)?)#');
}
if (!defined('_RAINETTE_REGEXP_LIEU_WEATHER_ID')) {
	/**
	 * Regexp permettant de reconnaître un lieu au format Weather ID.
	 */
	define('_RAINETTE_REGEXP_LIEU_WEATHER_ID', '#[a-zA-Z]{4}\d{4}#i');
}


/**
 * Normalise les données issues du service dans un tableau standard aux index prédéfinis pour chaque mode.
 *
 * @param array  $configuration_service Configuration statique et utilisateur du service ayant retourné le flux de données.
 * @param string $mode                  Le type de données météorologiques demandé :
 *                                      - `conditions`, la valeur par défaut
 *                                      - `previsions`
 *                                      - `infos`
 * @param array  $flux                  Le tableau brut des données météorologiques issu de l'appel au service.
 * @param int    $periode               Valeur de 0 à n pour indiquer qu'on traite les données de prévisions d'une période horaire donnée
 *                                      ou -1 pour indiquer que l'on traite les données jour. La valeur maximale n dépend de la périodicité
 *                                      des prévisions, par exemple, elle vaut 0 pour une périodicité de 24h, 1 pour 12h...
 *
 * @return array Le tableau standardisé des données météorologiques du service pour la période spécifiée.
 */
function meteo_normaliser($configuration_service, $mode, $flux, $periode) {
	$tableau = array();

	include_spip('inc/filtres');
	if ($flux !== null) {
		// Lecture de la configuration technique des données pour le mode concerné.
		include_spip('inc/config');
		$configuration_mode = lire_config("rainette_config/${mode}/donnees", array());

		// Le service a renvoyé des données, on boucle sur les clés du tableau normalisé
		// Néanmoins, en fonction de la période fournie en argument on filtre les données uniquement
		// utiles à cette période:
		// - si période = -1 on traite les données jour
		// - si période > -1 on traite les données heure
		foreach (array_keys($configuration_mode) as $_donnee) {
			if ((($periode == -1)
				 and (empty($configuration_mode[$_donnee]['rangement'])
					  or ($configuration_mode[$_donnee]['rangement'] == 'jour')))
				or (($periode > -1) and ($configuration_mode[$_donnee]['rangement'] == 'heure'))
			) {
				if ($configuration_mode[$_donnee]['origine'] == 'service') {
					// La donnée est fournie par le service. Elle n'est jamais calculée par le plugin
					// Néanmoins, elle peut-être indisponible temporairement
					if ($cle_service = $configuration_service['donnees'][$_donnee]['cle']) {
						// La donnée est normalement fournie par le service car elle possède une configuration de clé
						// On traite le cas où le nom de la clé varie suivant le système d'unité choisi ou la langue.
						// La clé de base peut être vide, le suffixe contenant dès lors toute la clé.
						if (!empty($configuration_service['donnees'][$_donnee]['suffixe_unite'])) {
							$systeme_unite = $configuration_service['unite'];
							$id_suffixee = $configuration_service['donnees'][$_donnee]['suffixe_unite']['id_cle'];
							$cle_service[$id_suffixee] .= $configuration_service['donnees'][$_donnee]['suffixe_unite'][$systeme_unite];
						} elseif (!empty($configuration_service['donnees'][$_donnee]['suffixe_langue'])) {
							$langue = langue_determiner($configuration_service);
							$id_suffixee = $configuration_service['donnees'][$_donnee]['suffixe_langue']['id_cle'];
							$cle_service[$id_suffixee] .= $langue;
						}

						// On utilise donc la clé pour calculer la valeur du service.
						// Si la valeur est disponible on la stocke sinon on met la donnée à chaine vide pour
						// montrer l'indisponibilité temporaire.
						$donnee = '';
						$valeur_service = empty($cle_service)
							? $flux
							: table_valeur($flux, implode('/', $cle_service));
						if ($valeur_service !== '') {
							$typer = donnee_typer($configuration_mode[$_donnee]['type_php'], $_donnee);
							$valeur_typee = $typer ? $typer($valeur_service) : $valeur_service;

							// Vérification de la donnée en cours de traitement si une fonction idoine existe
							$verifier = "donnee_verifier_${_donnee}";
							if (!function_exists($verifier) or (function_exists($verifier) and $verifier($valeur_typee))) {
								$donnee = $valeur_typee;
							}
						}
					} else {
						// La donnée météo n'est jamais fournie par le service. On la positionne à null pour
						// la distinguer avec une donnée vide qui indique une indisponibilité temporaire.
						$donnee = null;
					}
				} else {
					// La données météo est en général calculée à posteriori par le plugin indépendamment
					// du service. On l'initialise temporairement à null car il arrive parfois que le calcul
					// dépende d'une donnée du service qui n'est pas fournie (index UV par exemple).
					$donnee = null;
				}

				$tableau[$_donnee] = $donnee;
			}
		}
	}

	return $tableau;
}

/**
 * Détermine, en fonction du type PHP configuré, la fonction à appliquer à la valeur d'une donnée.
 *
 * @param string $type_php Le type PHP ou chaine vide sinon
 * @param string $donnee   Correspond à l'index du tableau associatif standardisé comme `temperature`, `humidite`,
 *                         `precipitation`...
 *
 * @return string La fonction PHP (floatval, intval...) ou spécifique à appliquer à la valeur de la donnée.
 */
function donnee_typer($type_php, $donnee) {

	// Si aucun type PHP précisé on renvoie aucune fonction à appliquer.
	$fonction = '';

	if ($type_php) {
		// Détermination de la fonction à appliquer
		switch ($type_php) {
			case 'float':
				$fonction = 'floatval';
				break;
			case 'int':
				$fonction = 'intval';
				break;
			case 'string':
				$fonction = 'strval';
				break;
			case 'date':
				$fonction = 'donnee_formater_date';
				break;
			case 'heure':
				$fonction = 'donnee_formater_heure';
				break;
			case 'mixed':
				$fonction = '';
				break;
			default:
				$fonction = '';
		}
	}

	return $fonction;
}

/**
 * Formate une date numérique ou sous une autre forme en une date au format `Y-m-d H:i:s`.
 *
 * @param string $donnee Correspond à un index du tableau associatif standardisé à formater en date standard.
 *
 * @return string Date au format `Y-m-d H:i:s`.
 */
function donnee_formater_date($donnee) {
	if (is_numeric($donnee)) {
		$date = date('Y-m-d H:i:s', $donnee);
	} else {
		$date = date_create($donnee);
		if (!$date) {
			$elements_date = explode(' ', $donnee);
			array_pop($elements_date);
			$donnee = implode(' ', $elements_date);
			$date = date_create($donnee);
		}
		$date = date_format($date, 'Y-m-d H:i:s');
	}

	return $date;
}

/**
 * Formate une heure numérique ou sous une autre forme en une heure au format `H:i`.
 *
 * @param string $donnee Correspond à un index du tableau associatif standardisé à formater en heure standard.
 *
 * @return string Heure au format `H:i`.
 */
function donnee_formater_heure($donnee) {
	if (is_numeric($donnee)) {
		$taille = strlen($donnee);
		if ($taille < 3) {
			$m = '00';
			$h = $donnee;
		} else {
			$m = substr($donnee, -2);
			$h = strlen($donnee) == 3
				? substr($donnee, 0, 1)
				: substr($donnee, 0, 2);
		}
		$heure = "${h}:${m}";
	} else {
		$heure = $donnee;
	}

	return $heure;
}

/**
 * Vérifie que la valeur de l'indice UV est acceptable.
 *
 * @param int $valeur Valeur de l'indice UV à vérifier. Un indice UV est toujours compris entre 0 et 16, bornes comprises.
 *
 * @return bool `true` si la valeur est acceptable, `false` sinon.
 */
function donnee_verifier_indice_uv($valeur) {
	$est_valide = true;
	if (($valeur < 0) or ($valeur > 16)) {
		$est_valide = false;
	}

	return $est_valide;
}

/**
 * @param $erreur
 * @param $lieu
 * @param $mode
 * @param $periodicite
 * @param $service
 * @param $configuration
 *
 * @return array
 */
function  erreur_normaliser_extras($erreur, $lieu, $mode, $periodicite, $service, $configuration = array()) {
	$extras = array(
		'credits'           => array(),
		'config'            => array(
			'nom_service' => $service
		),
		'lieu'              => $lieu,
		'mode'              => $mode,
		'periodicite_cache' => $periodicite,
		'service'           => $service,
		'erreur'            => $erreur
	);

	// On complète les informations extras uniquement si le service est connu actif ou non.
	if ($erreur['type'] != 'service_inconnu') {
		if (!$configuration) {
			// Acquérir la configuration statique du service (periode, format, données...)
			include_spip("services/${service}");
			$configurer = "${service}_service2configuration";
			$configuration = $configurer($mode);
		}

		// On prépare un contexte extras pour traiter les erreurs du modèle de façon standard comme celles
		// renvoyée par le chargement des données.
		$extras['credits'] = $configuration['credits'];
		$extras['config'] = array_merge(
			parametrage_normaliser($service, $configuration['defauts']),
			array('source'      => configuration_donnees_normaliser($mode, $configuration['donnees'])),
			array('nom_service' => $configuration['nom'])
		);
	}

	return $extras;
}

/**
 * @param $erreur
 * @param $lieu
 * @param $mode
 * @param $modele
 * @param $service
 * @param $nom_service
 *
 * @return array
 */
function erreur_formater_texte($erreur, $lieu, $mode, $modele, $service, $nom_service) {
	$texte = array('principal' => '', 'conseil' => '', 'service' => '');

	$type_erreur = $erreur['type'];
	switch ($type_erreur) {
		// Cas d'erreur lors du traitement de la requête par le plugin
		case 'url_indisponible':
		case 'analyse_xml':
		case 'analyse_json':
			// Cas d'erreur où le service renvoie aucune donnée sans pour autant monter une erreur.
		case 'aucune_donnee':
			// Cas d'erreur où le nombre de requêtes maximal a été atteint.
			$texte['principal'] .= _T("rainette:erreur_${type_erreur}", array('service' => $nom_service));
			$texte['conseil'] .= _T('rainette:erreur_conseil_equipe');
			break;
		// Cas d'erreur renvoyé par le service lui-même
		case 'reponse_service':
			if (!empty($erreur['service']['code'])) {
				$texte['service'] .= $erreur['service']['code'];
			}
			if (!empty($erreur['service']['message'])) {
				$texte['service'] .= ($texte['service'] ? ' - ' : '') . $erreur['service']['message'];
			}
			$texte['principal'] .= _T("rainette:erreur_${type_erreur}_${mode}", array('service' => $nom_service, 'lieu' => $lieu));
			$texte['conseil'] .= _T('rainette:erreur_conseil_service');
			break;
		// Cas d'erreur où le nombre de requêtes maximal a été atteint.
		case 'limite_service':
			$texte['principal'] .= _T("rainette:erreur_${type_erreur}", array('service' => $nom_service));
			$texte['conseil'] .= _T('rainette:erreur_conseil_limite');
			break;
		// Cas d'erreur du à une mauvaise utilisation des modèles
		case 'modele_periodicite':
			$texte['principal'] .= _T("rainette:erreur_${type_erreur}", array('modele' => $modele));
			$texte['conseil'] .= _T('rainette:erreur_conseil_periodicite');
			break;
		case 'modele_service':
			$texte['principal'] .= _T("rainette:erreur_${type_erreur}", array('modele' => $modele, 'service' => $nom_service));
			$texte['conseil'] .= _T('rainette:erreur_conseil_modele_changer');
			break;
		case 'modele_inutilisable':
			$texte['principal'] .= _T("rainette:erreur_${type_erreur}", array('modele' => $modele));
			$texte['conseil'] .= _T('rainette:erreur_conseil_modele_expliciter');
			break;
		case 'service_inactif':
			$texte['principal'] .= _T("rainette:erreur_${type_erreur}", array('service' => $nom_service));
			$texte['conseil'] .= _T('rainette:erreur_conseil_service_changer');
			break;
		case 'service_inconnu':
			$texte['principal'] .= _T("rainette:erreur_${type_erreur}", array('service' => $service));
			$texte['conseil'] .= _T('rainette:erreur_conseil_service_verifier');
			break;
	}

	return $texte;
}

/**
 * @param $type_modele
 * @param $service
 *
 * @return int
 */
function periodicite_determiner($type_modele, $service) {

	// Périodicité initialisée à "non trouvée"
	$periodicite = 0;

	// Lecture de la configuration technique des compatibilités type de modèle / périodicités
	include_spip('inc/config');
	$periodicites_modele = lire_config("rainette_config/modele_periodicites/${type_modele}", array());

	if ($periodicites_modele) {
		// Acquérir la configuration statique du service pour connaître les périodicités horaires supportées
		// pour le mode prévisions.
		include_spip("services/${service}");
		$configurer = "${service}_service2configuration";
		$configuration = $configurer('previsions');
		$periodicites_service = array_keys($configuration['periodicites']);

		// Déterminer la première périodicité compatible
		foreach ($periodicites_modele as $_periodicite_modele) {
			if (in_array($_periodicite_modele, $periodicites_service)) {
				$periodicite = $_periodicite_modele;
				break;
			}
		}
	}

	return $periodicite;
}

/**
 * @param $type_modele
 * @param $periodicite
 *
 * @return bool
 */
function periodicite_est_compatible($type_modele, $periodicite) {

	// Initialisation de la compatibilité à "non compatible".
	$compatible = false;

	// Lecture de la configuration technique des compatibilités type de modèle / périodicités
	include_spip('inc/config');
	$periodicites_modele = lire_config("rainette_config/modele_periodicites/${type_modele}", array());

	if (in_array($periodicite, $periodicites_modele)) {
		$compatible = true;
	}

	return $compatible;
}

/**
 * Construit le tableau du cache en fonction du service, du lieu, du type de données, de la langue utilisée par le site
 * et de l'unité des données.
 *
 * @param string $lieu
 *                                      Lieu pour lequel on requiert le nom du cache.
 * @param string $mode
 *                                      Type de données météorologiques. Les valeurs possibles sont `infos`, `conditions` ou `previsions`.
 * @param int    $periodicite
 *                                      La périodicité horaire des prévisions :
 *                                      - `24`, `12`, `6`, `3` ou `1`, pour le mode `previsions`
 *                                      - `0`, pour les modes `conditions` et `infos`
 * @param array  $configuration_service
 *                                      Configuration complète du service, statique et utilisateur. Contient l'unité choisie pour les données.
 *
 * @return array
 *               Chemin complet du fichier cache.
 */
function cache_normaliser($lieu, $mode, $periodicite, $configuration_service) {

	// Identification de la langue du resume.
	$cache = array();

	// Le cache est stocké dans un sous-dossier au nom du service
	$cache['sous_dossier'] = $configuration_service['alias'];

	// Composants obligatoires
	// -- Le nom du lieu normalisé (sans espace et dont tous les caractères non alphanumériques sont remplacés par un tiret)
	$lieu_normalise = lieu_normaliser($lieu);
	$cache['lieu'] = str_replace(array(' ', ',', '+', '.', '/'), '-', $lieu_normalise);

	// -- Le nom du mode (infos, conditions ou previsions) accolé à la périodicité du cache pour les prévisions uniquement
	$cache['donnees'] = $mode . ($periodicite ? strval($periodicite) : '');

	// -- Identification de la langue du resume.
	$code_langue = langue_determiner($configuration_service);
	$cache['langage'] = strtolower($code_langue);

	// Composants facultatifs
	// -- Unité des données si le mode n'est pas infos.
	if ($mode != 'infos') {
		$cache['unite'] = $configuration_service['unite'];
	}

	// La durée de conservation par défaut est positionné pour le mode infos. Pour les autres modes il faut
	// la positionner spécifiquement car chaque service a sa propre récurrence.
	if ($mode != 'infos') {
		$cache['conservation'] = $configuration_service['periode_maj'];
	}

	return $cache;
}

/**
 * @param string $lieu
 * @param string $format_lieu
 *
 * @return string
 */
function lieu_normaliser($lieu, &$format_lieu = '') {
	$lieu_normalise = trim($lieu);

	if (preg_match(_RAINETTE_REGEXP_LIEU_WEATHER_ID, $lieu_normalise, $match)) {
		$format_lieu = 'weather_id';
		$lieu_normalise = $match[0];
	} elseif (preg_match(_RAINETTE_REGEXP_LIEU_COORDONNEES, $lieu_normalise, $match)) {
		$format_lieu = 'latitude_longitude';
		$lieu_normalise = "{$match[1]},{$match[2]}";
	} elseif (preg_match(_RAINETTE_REGEXP_LIEU_IP, $lieu_normalise, $match)) {
		$format_lieu = 'adresse_ip';
		$lieu_normalise = $match[0];
	} elseif (intval($lieu_normalise)) {
		$format_lieu = 'city_id';
	} else {
		$format_lieu = 'ville_pays';
		// On détermine la ville et éventuellement le pays (ville[,pays])
		// et on élimine les espaces par un seul "+".
		$elements = explode(',', $lieu_normalise);
		$lieu_normalise = trim($elements[0]) . (!empty($elements[1]) ? ',' . trim($elements[1]) : '');
		$lieu_normalise = preg_replace('#\s+#', '+', $lieu_normalise);
	}

	return $lieu_normalise;
}

/**
 * Détermine la langue d'affichage du résumé météorologique.
 *
 * Les services de Rainette peuvent renvoyer la traduction du résumé dans plusieurs langues.
 * Il est donc nécessaire de demander ce résumé dans la bonne langue si elle existe.
 *
 * @param $configuration_service
 *
 * @return mixed
 */
function langue_determiner($configuration_service) {

	// On détermine la "bonne langue" à utiliser: on choisit soit celle de la page en cours
	// soit celle en cours pour l'affichage.
	$langue_spip = !empty($GLOBALS['lang']) ? $GLOBALS['lang'] : $GLOBALS['spip_lang'];

	// On cherche d'abord si le service fournit la langue utilisée par le site.
	// -- Pour cela on utilise la configuration du service qui fournit un tableau des langues disponibles
	//    sous le format [code de langue du service] = code de langue spip.
	$langue_service = array_search($langue_spip, $configuration_service['langues']['disponibles']);

	if ($langue_service === false) {
		// Lecture de la configuration technique des langues alternatives.
		include_spip('inc/config');
		$langues_alternatives = lire_config("rainette_config/langues_alternatives", array());

		// La langue utilisée par SPIP n'est pas supportée par le service.
		// -- On cherche si il existe une langue SPIP utilisable meilleure que la langue par défaut du service.
		// -- Pour ce faire on a défini pour chaque code de langue spip, un ou deux codes de langue SPIP à utiliser
		//    en cas d'absence de la langue concernée dans un ordre de priorité (index 0, puis index 1).
		$langue_service = $configuration_service['langues']['defaut'];
		if ($langues_alternatives[$langue_spip]) {
			foreach ($langues_alternatives[$langue_spip] as $_langue_alternative) {
				$langue_service = array_search($_langue_alternative, $configuration_service['langues']['disponibles']);
				if ($langue_service !== false) {
					break;
				}
			}
		}
	}

	// Aucune langue ne correspond véritablement, on choisit donc la langue configurée par défaut.
	if ($langue_service === false) {
		$langue_service = $configuration_service['langues']['defaut'];
	}

	return $langue_service;
}

/**
 * @param $mode
 * @param $configuration
 *
 * @return array
 */
function configuration_donnees_normaliser($mode, $configuration) {

	// On renvoie un tableau vide en cas d'erreur
	$configuration_normalisee = array();

	// Lecture de la configuration technique des données pour le mode concerné.
	include_spip('inc/config');
	$configuration_mode = lire_config("rainette_config/${mode}/donnees", array());

	foreach ($configuration_mode as $_donnee => $_configuration) {
		if ($_configuration['origine'] == 'service') {
			$configuration_normalisee[$_donnee] = !empty($configuration[$_donnee]['cle']);
		}
	}

	return $configuration_normalisee;
}

/**
 * @param $service
 * @param $configuration_defaut
 *
 * @return mixed
 */
function parametrage_normaliser($service, $configuration_defaut) {

	// On récupère la configuration utilisateur
	include_spip('inc/config');
	$configuration_utilisateur = lire_config("rainette/${service}", array());

	// On complète la configuration avec des valeurs par défaut si nécessaire.
	foreach ($configuration_defaut as $_cle => $_valeur) {
		if (!isset($configuration_utilisateur[$_cle])) {
			$configuration_utilisateur[$_cle] = $_valeur;
		}
	}

	return $configuration_utilisateur;
}

/**
 * @param int|string $code_meteo
 * @param string     $theme
 * @param array      $transcodage
 * @param int        $periode
 *
 * @return string
 */
function icone_weather_normaliser($code_meteo, $theme, $transcodage = array(), $periode = 0) {

	// Si le transcodage échoue ou que le code weather est erroné on renvoie toujours N/A.
	$icone = 'na';

	// Transcodage en code weather.com.
	$code = is_string($code_meteo) ? strtolower($code_meteo) : intval($code_meteo);
	if ($transcodage) {
		// Service différent de weather.com
		if (array_key_exists($code, $transcodage) and isset($transcodage[$code][$periode])) {
			$icone = strval($transcodage[$code][$periode]);
		}
	} elseif (($code >= 0) and ($code < 48)) {
		// Service weather.com
		$icone = strval($code);
	}

	// Construction du chemin complet de l'icone.
	$chemin = icone_local_normaliser("${icone}.png", 'weather', $theme);

	return $chemin;
}

/**
 * @param string $icone
 * @param string $service
 * @param string $theme
 * @param string $periode
 *
 * @return string
 */
function icone_local_normaliser($icone, $service, $theme = '', $periode = '') {

	// On initialise le dossier de l'icone pour le service demandé.
	$chemin = "themes/${service}";
	// Si on demande un thème il faut créer le sous-dossier.
	if ($theme) {
		$chemin .= "/${theme}";
	}
	// Si le service gère des icones suivant le jour ou la nuit il faut ajouter le sous-dossier concerné.
	if ($periode) {
		$chemin .= "/${periode}";
	}
	// On finalise le chemin complet avec le nom de l'icone sauf si on ne veut que le dossier.
	if ($icone) {
		$chemin .= "/${icone}";
	}

	return $chemin;
}

/**
 * @param string $service
 *
 * @return string
 */
function service_est_indisponible($service) {
	$services = rainette_lister_services('tableau', false);
	if (!array_key_exists($service, $services)) {
		// L'identifiant du service est erroné.
		$erreur = 'service_inconnu';
	} elseif (!$services[$service]['actif']) {
		// Le service est inactivé.
		$erreur = 'service_inactif';
	} else {
		$erreur = '';
	}

	return $erreur;
}
